/*
 * Copyright (C) 2008 - Neil Kettle
 *
 * This file is part of equivset.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>

#include <cudd/cudd.h>
#include <cudd/cuddInt.h>

#include "list.h"
#include "lookup.h"

#include "equiv.h"

static struct equiv_list *
unify_compress (struct equiv_list *src, struct lookup_list *theta)
{
  struct equiv_list *r, *rt;
  int idx;

  if (LIST_EMPTY (src))
    return (NULL);

  rt = equiv_list_alloc (128);

  for (idx = 0; idx < src->num_items; idx++)
    {
      struct equiv_list_item *cur;
      struct lookup_pair *ptr;

      cur = &src->items[idx];

      if (cur->p <= 1)
        equiv_list_insert (rt, cur->idx, cur->p, cur->n);
      else if (LOOKUP_NOT_EMPTY (theta))
        {
          ptr = lookup_search_value (theta, cur->p);
          if (ptr != NULL && ptr->used > 1)
            equiv_list_insert (rt, cur->idx, cur->p, cur->n);
        }
    }

  if (LIST_EMPTY (rt))
    r = NULL;
  else
    r = equiv_list_copy (rt);

  equiv_list_free (rt);

  return (r);
}

struct equiv_list *
anti_unify_equiv (struct equiv_list *alist, struct equiv_list *blist)
{
  struct equiv_list *cr, *r;
  struct lookup_list *theta;
  int a_idx, b_idx;

  theta = lookup_list_alloc (128);
  r = equiv_list_alloc (128);

  a_idx = 0;
  b_idx = 0;

  while (a_idx < alist->num_items && b_idx < blist->num_items)
    {
      struct equiv_list_item *cur_l1, *cur_l2;

      cur_l1 = &alist->items[a_idx];
      cur_l2 = &blist->items[b_idx];

      if (cur_l1->idx < cur_l2->idx)
        a_idx++;
      else if (cur_l1->idx > cur_l2->idx)
        b_idx++;
      else
        {
          if (cur_l1->p == cur_l2->p && cur_l1->p <= 1)
            equiv_list_insert (r, cur_l1->idx, cur_l1->p, cur_l1->n);
          else
            {
              struct lookup_pair *lookup;
              int p, n;

#ifdef DEBUG
              printf ("searching for keys: %d:%d, %d:%d\n", cur_l1->p,
                      cur_l2->p, cur_l1->n, cur_l2->n);
#endif

              lookup = lookup_search_keypair (theta, cur_l1->p, cur_l2->p,
                                              cur_l1->n, cur_l2->n);
              if (lookup == NULL)
                {
#ifdef DEBUG
                  printf
                    ("not found pair in lookup mapping %d %d:%d %d:%d \n",
                     cur_l1->idx, cur_l1->p, cur_l2->p, cur_l1->n, cur_l2->n);

                  printf ("inserting key: %d:%d %d:%d\n", cur_l1->p,
                          cur_l2->p, cur_l1->n, cur_l2->n);
#endif

                  lookup_insert_pair (theta, cur_l1->p, cur_l2->p,
                                      cur_l1->n, cur_l2->n);
                  p = theta->items[theta->num_items - 1].keys[0].value;
                  n = theta->items[theta->num_items - 1].keys[1].value;
                }
              else
                {
#ifdef DEBUG
                  printf ("found pair in lookup %d %d:%d -> %d:%d\n",
                          cur_l1->idx, cur_l1->p, cur_l2->p,
                          lookup->keys[0].value, lookup->keys[1].value);
#endif
                  lookup->used++;
                  p = lookup->keys[0].value;
                  n = lookup->keys[1].value;
                }

              equiv_list_insert (r, cur_l1->idx, p, n);
            }

          a_idx++;
          b_idx++;
        }
    }

  cr = unify_compress (r, theta);

  lookup_list_free (theta);
  equiv_list_free (r);

  return (cr);
}
