/*
 * Copyright (C) 2008 - Neil Kettle
 *
 * This file is part of equivset.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>

#include <string.h>

#include "list.h"

static struct equiv_list *_equiv_list_init (int);

static struct equiv_list *
_equiv_list_init (int num_items)
{
  struct equiv_list *ptr;

  ptr = malloc (sizeof (struct equiv_list) +
                (sizeof (struct equiv_list_item) * num_items));
  if (ptr == NULL)
    {
      fprintf (stderr, "_equiv_list_init: failed allocating %d-bytes\n",
               sizeof (struct equiv_list) +
                (sizeof (struct equiv_list_item) * num_items));
      exit (EXIT_FAILURE);
    }

  ptr->num_items = num_items;
  ptr->max_items = num_items;

  return (ptr);
}

struct equiv_list *
equiv_list_alloc (int num_items)
{
  struct equiv_list *ptr;

  ptr = malloc (sizeof (struct equiv_list) +
                (sizeof (struct equiv_list_item) * num_items));
  if (ptr == NULL)
    {
      fprintf (stderr, "_equiv_list_alloc: failed allocating %d-bytes\n",
               sizeof (struct equiv_list) +
                (sizeof (struct equiv_list_item) * num_items));
      exit (EXIT_FAILURE);
    }

  ptr->num_items = 0;
  ptr->max_items = num_items;

  return (ptr);
}

void
equiv_list_insert (struct equiv_list *src, int idx, int p, int n)
{
  int nitems;

  if (src->num_items + 1 > src->max_items)
    {
      fprintf (stderr, "equiv_list_insert: src->num_items + 1 > src->max_items\n");
      exit (EXIT_FAILURE);
    }

  nitems = src->num_items;
  src->items[nitems].idx = idx;
  src->items[nitems].p = p;
  src->items[nitems].n = n;
  src->num_items++;

  return;
}

struct equiv_list *
equiv_list_init_empty (void)
{
  return (_equiv_list_init (0));
}

struct equiv_list *
equiv_list_init_single (int idx, int p, int n)
{
  struct equiv_list *ptr;

  ptr = _equiv_list_init (1);
  ptr->items[0].idx = idx;
  ptr->items[0].p = p;
  ptr->items[0].n = n;

  return (ptr);
}

struct equiv_list *
equiv_list_copy_insert (struct equiv_list *src, int idx, int p, int n)
{
  struct equiv_list *ptr;

  ptr = _equiv_list_init (src->num_items + 1);
  memcpy (&ptr->items[1], &src->items,
          sizeof (struct equiv_list_item) * src->num_items);

  ptr->items[0].idx = idx;
  ptr->items[0].p = p;
  ptr->items[0].n = n;

  return (ptr);
}

struct equiv_list *
equiv_list_copy (struct equiv_list *src)
{
  struct equiv_list *ptr;

  ptr = _equiv_list_init (src->num_items);
  memcpy (&ptr->items, &src->items,
          sizeof (struct equiv_list_item) * src->num_items);

  return (ptr);
}

void
equiv_list_dump (struct equiv_list *src)
{
  int i;

  printf ("num_items: %d\n", src->num_items);
  for (i = 0; i < src->num_items; i++)
    printf ("index: %d, pos: %d, neg: %d\n",
            src->items[i].idx, src->items[i].p, src->items[i].n);

  printf ("\n");

  return;
}
