/*
 * Copyright (C) 2008 - Neil Kettle
 *
 * This file is part of equivset.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>

#include "lookup.h"

static int newvar = 2;

static int
_lookup_get_newvar (void)
{
  return (newvar++);
}

struct lookup_list *
lookup_list_alloc (int num_items)
{
  struct lookup_list *ptr;

  ptr = malloc (sizeof (struct lookup_list) +
                (sizeof (struct lookup_pair) * num_items));
  if (ptr == NULL)
    {
      fprintf (stderr, "lookup_list_alloc: failed allocating %d-bytes\n",
               sizeof (struct lookup_list) +
                (sizeof (struct lookup_pair) * num_items));
      exit (EXIT_FAILURE);
    }

  ptr->num_items = 0;
  ptr->max_items = num_items;

  return (ptr);
}

struct lookup_pair *
lookup_search_keypair (struct lookup_list *list,
                       int k1_p, int k2_p, int k1_n, int k2_n)
{
  int i;

  for (i = 0; i < list->num_items; i++)
    {
      if (list->items[i].keys[0].p == k1_p &&
          list->items[i].keys[0].n == k2_p &&
          list->items[i].keys[1].p == k1_n &&
          list->items[i].keys[1].n == k2_n)
        return (&list->items[i]);
    }

  return (NULL);
}

struct lookup_pair *
lookup_search_value (struct lookup_list *list,
                     int value)
{
  int i;

  for (i = 0; i < list->num_items; i++)
    {
      if (list->items[i].keys[0].value == value ||
          list->items[i].keys[1].value == value)
        return (&list->items[i]);
    }

  return (NULL);
}

void
lookup_insert_pair (struct lookup_list *list,
                    int k1_p, int k2_p, int k1_n, int k2_n)
{
  int nitems;

  if (list->num_items + 1 > list->max_items)
    {
      fprintf (stderr, "lookup_insert_pair: list->num_items + 1 > list->max_items\n");
      exit (EXIT_FAILURE);
    }

  nitems = list->num_items;
  list->items[nitems].used = 1;
  list->items[nitems].keys[0].p = k1_p;
  list->items[nitems].keys[0].n = k2_p;
  list->items[nitems].keys[0].value = _lookup_get_newvar ();
  list->items[nitems].keys[1].p = k1_n;
  list->items[nitems].keys[1].n = k2_n;
  list->items[nitems].keys[1].value = _lookup_get_newvar ();
  list->num_items++;

  return;
}
